/*
	PlastererUtil
	Copyright (C) 2003 ONO,Masayuki.<mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "PlastererUtil.h"

#include <qtabwidget.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qwhatsthis.h>
#include <qcheckbox.h>
#include <qmessagebox.h>
#include <qpe/config.h>
#include <qpe/qcopenvelope_qws.h>
#include <qtimer.h>
#include <sl/sldlgwait.h>

#include "../applet/plastererdefine.h"
#include <sl/slfileselector.h>
#include <sl/slcolorselector.h>

ImageBrowser::ImageBrowser( QWidget* parent, 
							const char* name, bool modal, WFlags fl )
: QDialog( parent, name, modal, fl )
{
	setWState( WState_Reserved1 );

	if ( !name )
	{
		setName( "ImageBrowser" );
	}
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5,
												sizePolicy().hasHeightForWidth() ) );
	QString caption( tr("Select a Image") );
	setCaption( caption );

	BaseLayout = new QVBoxLayout( this ); 
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );

	ExeBrowser = new SlFileSelector( QPEApplication::documentDir(), "image/*", this, "ExeBrowser" );
	ExeBrowser->setFocusMode( SlFileSelector::ByTab );
	BaseLayout->addWidget( ExeBrowser );

	connect( ExeBrowser, SIGNAL( keyPressed( QKeyEvent*,bool& ) ),
				this, SLOT( BrowserKeyPressed( QKeyEvent*, bool& ) ) );

	showMaximized();
	ExeBrowser->setSelected();
}

ImageBrowser::~ImageBrowser()
{
}

void ImageBrowser::closeEvent( QCloseEvent *e )
{
	e->accept();
}

void ImageBrowser::accept()
{
	m_sPath = ExeBrowser->currentFile();
	if ( m_sPath[0] == '.' )
	{
		m_sPath = m_sPath.mid( 1 );
	}
	QFileInfo fi( m_sPath );
	if ( fi.isFile() )
	{
		emit done( QDialog::Accepted );
	}
	close();
}

void ImageBrowser::reject()
{
	close();
}

QString ImageBrowser::getPath()
{
	return m_sPath;
}

PlastererUtil::PlastererUtil()
:PlastererUtilBase()
{
	m_pWaitDlg = NULL;
	m_bAccepted = FALSE;
	m_tComboValue[ 0 ].nValue = ILV_B;
	m_tComboValue[ 0 ].sLabel = tr( "Black" );
	m_tComboValue[ 1 ].nValue = ILV_W;
	m_tComboValue[ 1 ].sLabel = tr( "White" );
	m_tComboValue[ 2 ].nValue = ILV_BBW;
	m_tComboValue[ 2 ].sLabel = tr( "Black(WhiteBack)" );
	m_tComboValue[ 3 ].nValue = ILV_WWS;
	m_tComboValue[ 3 ].sLabel = tr( "White(WithShadow)" );
	m_tComboValue[ 4 ].nValue = ILV_WWR;
	m_tComboValue[ 4 ].sLabel = tr( "White(WithRim)" );
	m_tComboValue[ 5 ].nValue = ILV_NON;
	m_tComboValue[ 5 ].sLabel = tr( "None" );

	setWState( WState_Reserved1 );
	QString caption;
	caption = tr( "Plasterer" );
	caption += " Ver.";
	caption += _version_;
	setCaption( caption );

	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	ImagePathVEdit->setText( cfg.readEntry( KEY_VFN, "" ) );
	ImagePathHEdit->setText( cfg.readEntry( KEY_HFN, "" ) );
	m_strVFN = ImagePathVEdit->text();
	m_strHFN = ImagePathHEdit->text();

	ScalingVCheck->setChecked( cfg.readBoolEntry( KEY_VFT, FALSE ) );
	ScalingHCheck->setChecked( cfg.readBoolEntry( KEY_HFT, FALSE ) );
	m_bVFT = ScalingVCheck->isChecked();
	m_bHFT = ScalingHCheck->isChecked();

	m_strVBC = cfg.readEntry( KEY_VBC, "0,0,0" );
	{
		QStringList tmpList = QStringList::split( ",", m_strVBC );
		selectedBackColorV( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );
	}
	
	m_strHBC = cfg.readEntry( KEY_HBC, "0,0,0" );
	{
		QStringList tmpList = QStringList::split( ",", m_strHBC );
		selectedBackColorH( qRgb( tmpList[0].toInt(),
									tmpList[1].toInt(),
									tmpList[2].toInt() ) );
	}

	if ( cfg.readBoolEntry( KEY_VTM, TRUE ) )
	{
		m_bVTM = TRUE;
		TilingVRadio->setChecked( TRUE );
		CenteringVRadio->setChecked( FALSE );
		ScalingVCheck->setChecked( FALSE );
		ScalingVCheck->setEnabled( FALSE );
		BackColorVButton->setEnabled( FALSE );
	}
	else
	{
		m_bVTM = FALSE;
		TilingVRadio->setChecked( FALSE );
		CenteringVRadio->setChecked( TRUE );
	}

	if ( cfg.readBoolEntry( KEY_HTM, TRUE ) )
	{
		m_bHTM = TRUE;
		TilingHRadio->setChecked( TRUE );
		CenteringHRadio->setChecked( FALSE );
		ScalingHCheck->setChecked( FALSE );
		ScalingHCheck->setEnabled( FALSE );
		BackColorHButton->setEnabled( FALSE );
	}
	else
	{
		m_bHTM = FALSE;
		TilingHRadio->setChecked( FALSE );
		CenteringHRadio->setChecked( TRUE );
	}

	if ( cfg.readBoolEntry( KEY_SVH, TRUE ) )
	{
		m_bSVH = TRUE;
		UseSameImageVHCheck->setChecked( TRUE );
		Tabs->setTabEnabled( TabH, FALSE );
	}
	else
	{
		m_bSVH = FALSE;
		UseSameImageVHCheck->setChecked( FALSE );
		Tabs->setTabEnabled( TabH, TRUE );
	}
	
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		IconLabelVCombo->insertItem( m_tComboValue[ nCnt ].sLabel, nCnt );
		IconLabelHCombo->insertItem( m_tComboValue[ nCnt ].sLabel, nCnt );
	}
	m_nDefVLB = m_nVLB = cfg.readNumEntry( KEY_VLB, 0 );
	m_nDefHLB = m_nHLB = cfg.readNumEntry( KEY_HLB, 0 );
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].nValue == m_nDefVLB )
		{
			IconLabelVCombo->setCurrentItem( nCnt );
		}
	}
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].nValue == m_nDefHLB )
		{
			IconLabelHCombo->setCurrentItem( nCnt );
		}
	}

	connect( ApplyButton, SIGNAL( clicked() ), this, SLOT( clickedApplyButton() ) );
	connect( OkButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( CancelButton, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( ImagePathVButton, SIGNAL( clicked() ), this, SLOT( clickedImagePathVButton() ) );
	connect( ImagePathHButton, SIGNAL( clicked() ), this, SLOT( clickedImagePathHButton() ) );
	connect( BackColorVButton, SIGNAL( clicked() ), this, SLOT( clickedBackColorVButton() ) );
	connect( BackColorHButton, SIGNAL( clicked() ), this, SLOT( clickedBackColorHButton() ) );
	connect( ImageAlignVGroup, SIGNAL( clicked( int ) ), 
						this, SLOT( clickedImageAlignVGroup( int ) ) );
	connect( ImageAlignHGroup, SIGNAL( clicked( int ) ), 
						this, SLOT( clickedImageAlignHGroup( int ) ) );
	connect( UseSameImageVHCheck, SIGNAL( clicked() ), this, SLOT( clickedUseSameImageCheck() ) );
	connect( IconLabelVCombo, SIGNAL( activated(const QString&) ),
	                this, SLOT( iconLabelVChanged(const QString&) ) );
	connect( IconLabelHCombo, SIGNAL( activated(const QString&) ),
	                this, SLOT( iconLabelHChanged(const QString&) ) );
}

PlastererUtil::~PlastererUtil()
{
}

void PlastererUtil::closeEvent( QCloseEvent *e )
{
	if ( ! m_bAccepted )
	{
		m_pWaitDlg = new SlDlgWait( this, "Plasterer\n now Making Images", TRUE, FALSE );
		m_pWaitDlg->move( QPoint( ( QPEApplication::desktop()->width() - m_pWaitDlg->width() ) / 2 ,
							( QPEApplication::desktop()->height() - m_pWaitDlg->height() ) / 2 ) );
		m_pWaitDlg->exec();

		Config cfg( CNF_P );
		cfg.setGroup( SEC_WP );

		cfg.writeEntry( KEY_VFN, m_strVFN );
		cfg.writeEntry( KEY_HFN, m_strHFN );

		cfg.writeEntry( KEY_VFT, m_bVFT );
		cfg.writeEntry( KEY_HFT, m_bHFT );

		cfg.writeEntry( KEY_VTM, m_bVTM );
		cfg.writeEntry( KEY_HTM, m_bHTM );

		cfg.writeEntry( KEY_VBC, m_strVBC );
		cfg.writeEntry( KEY_HBC, m_strHBC );

		cfg.writeEntry( KEY_VLB, m_nDefVLB );
		cfg.writeEntry( KEY_HLB, m_nDefHLB );

		cfg.writeEntry( KEY_SVH, m_bSVH );

		QTimer::singleShot( 1, this, SLOT( applyWallpaper() ) );
	}
	e->accept();
}

void PlastererUtil::accept()
{
	m_bAccepted = TRUE;
	clickedApplyButton();
	close();
}

void PlastererUtil::reject()
{
	close();
}

void PlastererUtil::clickedUseSameImageCheck()
{
	if ( UseSameImageVHCheck->isChecked() )
	{
		Tabs->setTabEnabled( TabH, FALSE );
	}
	else
	{
		Tabs->setTabEnabled( TabH, TRUE );
	}
}

void PlastererUtil::clickedImageAlignVGroup( int id )
{
	if ( id == idTiling )
	{
		ScalingVCheck->setChecked( FALSE );
		ScalingVCheck->setEnabled( FALSE );
		BackColorVButton->setEnabled( FALSE );
	}
	else
	{
		ScalingVCheck->setEnabled( TRUE );
		BackColorVButton->setEnabled( TRUE );
	}
}

void PlastererUtil::clickedImageAlignHGroup( int id )
{
	if ( id == idTiling )
	{
		ScalingHCheck->setChecked( FALSE );
		ScalingHCheck->setEnabled( FALSE );
		BackColorHButton->setEnabled( FALSE );
	}
	else
	{
		ScalingHCheck->setEnabled( TRUE );
		BackColorHButton->setEnabled( TRUE );
	}
}

void PlastererUtil::iconLabelVChanged( const QString& str )
{
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].sLabel == str )
		{
			m_nVLB = m_tComboValue[ nCnt ].nValue;
			break;
		}
	}
}

void PlastererUtil::iconLabelHChanged( const QString& str )
{
	for ( int nCnt = 0; nCnt < 6; nCnt ++ )
	{
		if ( m_tComboValue[ nCnt ].sLabel == str )
		{
			m_nHLB = m_tComboValue[ nCnt ].nValue;
			break;
		}
	}
}

void PlastererUtil::clickedApplyButton()
{
	if ( m_pWaitDlg )
	{
		return;
	}
	m_pWaitDlg = new SlDlgWait( this, "Plasterer\n now Making Images", TRUE, FALSE );
	m_pWaitDlg->move( QPoint( ( QPEApplication::desktop()->width() - m_pWaitDlg->width() ) / 2 ,
						( QPEApplication::desktop()->height() - m_pWaitDlg->height() ) / 2 ) );
	m_pWaitDlg->exec();

	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	cfg.writeEntry( KEY_SVH, UseSameImageVHCheck->isChecked() );

	cfg.writeEntry( KEY_VFN, ImagePathVEdit->text() );
	cfg.writeEntry( KEY_VFT, ScalingVCheck->isChecked() );
	cfg.writeEntry( KEY_VTM, TilingVRadio->isChecked() );
	
	QString tmpStr;
	tmpStr.sprintf( "%d,%d,%d", m_clrVBC.red(), m_clrVBC.green(), m_clrVBC.blue() );
	cfg.writeEntry( KEY_VBC, tmpStr );
	cfg.writeEntry( KEY_VLB, m_nVLB );

	if ( UseSameImageVHCheck->isChecked() )
	{
		cfg.writeEntry( KEY_HFN, ImagePathVEdit->text() );
		cfg.writeEntry( KEY_HFT, ScalingVCheck->isChecked() );
		cfg.writeEntry( KEY_HTM, TilingVRadio->isChecked() );
		cfg.writeEntry( KEY_HBC, tmpStr );
		cfg.writeEntry( KEY_HLB, m_nVLB );
	}
	else
	{
		cfg.writeEntry( KEY_HFN, ImagePathHEdit->text() );
		cfg.writeEntry( KEY_HFT, ScalingHCheck->isChecked() );
		cfg.writeEntry( KEY_HTM, TilingHRadio->isChecked() );
		tmpStr.sprintf( "%d,%d,%d", m_clrHBC.red(), m_clrHBC.green(), m_clrHBC.blue() );
		cfg.writeEntry( KEY_HBC, tmpStr );
		cfg.writeEntry( KEY_HLB, m_nHLB );
	}

	QTimer::singleShot( 1, this, SLOT( applyWallpaper() ) );
}

void PlastererUtil::applyWallpaper()
{
	makeImages();
	QCopEnvelope e( "QPE/Plasterer", "reload()" );
}

void PlastererUtil::clickedBackColorVButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackColorV( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, BackColorVButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void PlastererUtil::selectedBackColorV( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	BackColorVButton->setPixmap( tmpPixmap );
	m_clrVBC = clrT;
}

void PlastererUtil::clickedBackColorHButton()
{
	SlColorSelectorPopup* colorSelector = new SlColorSelectorPopup( FALSE, this );
	connect( colorSelector, SIGNAL( selected( QRgb ) ), this, SLOT( selectedBackColorH( QRgb ) ) );
	QPoint tPoint( width() / 2 - 20, BackColorHButton->geometry().top() );
	tPoint = mapToGlobal( tPoint );
	colorSelector->popup( tPoint );
}

void PlastererUtil::selectedBackColorH( QRgb rgb )
{
	QColor clrT(rgb);
		QFontMetrics fm( font() );
		QString strButtonCaption( tr("Back Color") );
		QPixmap tmpPixmap( fm.width( strButtonCaption ) + 16, fm.height() + 6 );
		tmpPixmap.fill( clrT );
		QPainter p( &tmpPixmap );
		p.setPen( QColor ( 0xFF & ~clrT.red(),
							0xFF & ~clrT.green(),
							0xFF & ~clrT.blue() ) );
		p.drawText( 9, fm.height() + 2, strButtonCaption );
		p.flush();
	BackColorHButton->setPixmap( tmpPixmap );
	m_clrHBC = clrT;
}

void PlastererUtil::clickedImagePathVButton()
{
	ImageBrowser dlg( this, "Image Selector", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		ImagePathVEdit->setText( dlg.getPath() );
	}
}

void PlastererUtil::clickedImagePathHButton()
{
	ImageBrowser dlg( this, "Image Selector", TRUE, WType_Modal );
	if ( dlg.exec() == QDialog::Accepted )
	{
		ImagePathHEdit->setText( dlg.getPath() );
	}
}

void PlastererUtil::makeImages()
{
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	QSize maxWidgetSize = QPEApplication::desktop()->size();
	if ( maxWidgetSize.height() < maxWidgetSize.width() )
	{
		maxWidgetSize = QSize( maxWidgetSize.height(), maxWidgetSize.width() );
	}

	QSize sizeWidget;

	int corWidth = cfg.readNumEntry( KEY_WCO, -1 );
	if ( corWidth < 0 )
	{
		corWidth = SlMisc::getScrollbarMetrics().width();
	}
	int corHeight = cfg.readNumEntry( KEY_HCO, -1 );

	//sizeWidget = QSize( maxWidgetSize.width() - SlMisc::getScrollbarMetrics().width(),
	//					(int) ( (double)maxWidgetSize.height() * 0.80 ) );
	sizeWidget = QSize( maxWidgetSize.width() - corWidth,
						corHeight >= 0 ? maxWidgetSize.height() - corHeight
										: (int) ( maxWidgetSize.height() * 0.80 ) );
	QImage imgV( cfg.readEntry( KEY_VFN, "" ) );
	makeImage( &imgV, cfg, sizeWidget, KEY_VFN );
	imgV.save( sImgPath + WF_V_IMG, "PNG" );

	//sizeWidget = QSize( (int) ( (double) maxWidgetSize.width() * 0.80 ),
	//					maxWidgetSize.height() - SlMisc::getScrollbarMetrics().width() );
	sizeWidget = QSize( corHeight >= 0 ? maxWidgetSize.width() - corHeight
										: (int) ( maxWidgetSize.width() * 0.80 ),
						maxWidgetSize.height() - corWidth );
	QImage imgH( cfg.readEntry( KEY_HFN, "" ) );
	makeImage( &imgH, cfg, sizeWidget, KEY_HFN );
	imgH.save( sImgPath + WF_H_IMG, "PNG" );

	if ( m_pWaitDlg )
	{
		delete m_pWaitDlg;
		m_pWaitDlg = NULL;
	}
}

void PlastererUtil::makeImage( QImage* img,
								Config& cfg,
								const QSize& rSizeDesktop,
								const QString keyFName )
{
	bool isAlignV = ( keyFName == KEY_VFN ) ? TRUE : FALSE;

	if ( ! cfg.readBoolEntry( isAlignV ? KEY_VTM : KEY_HTM, TRUE )
		&& ( ! img->isNull() )
		&& ( img->size() != QSize( 0, 0 ) ) )
	{
		QSize rSize;
		if ( rSizeDesktop.height() > rSizeDesktop.width() )
		{
			rSize = isAlignV ? rSizeDesktop : QSize( rSizeDesktop.height(), rSizeDesktop.width() );
		}
		else
		{
			rSize = isAlignV ? QSize( rSizeDesktop.height(), rSizeDesktop.width() ) : rSizeDesktop;
		}
		QSize sizeScaling = img->size();

		if ( cfg.readBoolEntry( isAlignV ? KEY_VFT : KEY_HFT, FALSE ) )
		{
			if ( isAlignV )
			{
				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );

				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );
			}
			else
			{
				sizeScaling.setHeight( (double)sizeScaling.height() 
								* ( (double)rSize.width() / (double)sizeScaling.width() ) );
				sizeScaling.setWidth( rSize.width() );

				sizeScaling.setWidth( (double)sizeScaling.width() 
								* ( (double)rSize.height() / (double)sizeScaling.height() ) );
				sizeScaling.setHeight( rSize.height() );
			}

			if ( img->size() != sizeScaling )
			{
				*img = img->smoothScale( sizeScaling.width(), sizeScaling.height() );
			}
		}

		QPoint drawP( ( rSize.width() - sizeScaling.width() ) / 2,
							( rSize.height() - sizeScaling.height() ) / 2 );
		QPixmap pixT;
		pixT.resize( rSize );
		QStringList tmpList = QStringList::split( ",", cfg.readEntry( isAlignV ? KEY_VBC : KEY_HBC, "0,0,0" ) );
		QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
		pixT.fill( tmpColor );
		QPainter p( &pixT );
		p.drawImage( drawP, *img );
		p.flush();
		*img = pixT.convertToImage();
	}
}

